@@ -10,8 +10,6 @@ import datatoolbox as dt
 dt.admin.switch_database_to_testing()
 
 def test_init():
-    dt.inventory()
-    dt.DBinfo()
     dt.sourceInfo()
     assert len(dt.sourceInfo()) == len(dt.core.DB.sources)
     
